import java.awt.image.*;
import java.io.*;
import java.util.*;

public class BitMapImageProducer implements ImageProducer {
  byte bitmap[];
  Vector consumers;
  int width;
  int height;
  ColorModel colorModel;

  public BitMapImageProducer(byte[] bitMaps, long offset,
			     int w, int h, int pixelsPerByte, boolean reversed) {
    width = w;
    height = h;
    int byteWidth = w/pixelsPerByte;
    byte data[] = new byte[byteWidth*h];
    consumers = new Vector();
    colorModel = (ColorModel) new DirectColorModel(8, 255, 255, 255);
    System.arraycopy(bitMaps, (int)offset, data, 0, data.length);
    
    byte fg = (byte)255;
    byte bg = (byte)0;
    if (reversed) { // if foreground and bg colors reversed
      fg = (byte)0;
      bg = (byte)255;
    }
    bitmap = new byte[w*h];
    byte initMask = 0;
    int shiftAmt = 0;
    int numShifts = 0;
    switch (pixelsPerByte) {
      case 8: initMask = (byte)1; shiftAmt = 1; break;
      case 4: initMask = (byte)3; shiftAmt = 2; break;
      case 2: initMask = (byte)15; shiftAmt = 4; break;
      case 1: initMask = (byte)255; shiftAmt = 8; break;
      default: System.err.println("invalid pixels-per-byte value");
               initMask = (byte)255; shiftAmt = 1; pixelsPerByte = 1; break;
    }
    for(int i=0; i<h; i++)
      for(int j=0; j<byteWidth; j++) {
	byte mask = initMask;
	for (int k=0; k<pixelsPerByte; k++) {
	  if ((mask & data[i*byteWidth+j]) > 0 || (mask==(byte)-128 &&
	       data[i*byteWidth+j]<(byte)0))
	    bitmap[i*w+j*pixelsPerByte+pixelsPerByte-1-k] = fg;
	  else {
	    bitmap[i*w+j*pixelsPerByte+pixelsPerByte-1-k] = bg;
	  }
	  mask = (byte) (mask << shiftAmt);
	}
      }
  }

  public void addConsumer(ImageConsumer  ic) {
    consumers.addElement(ic);
  }

  public boolean isConsumer(ImageConsumer  ic) {
    return consumers.contains(ic);
  }

  public void removeConsumer(ImageConsumer  ic) {
    if (isConsumer(ic))
      consumers.removeElement(ic);
  }

  public void requestTopDownLeftRightResend(ImageConsumer ic) {
    ic.setHints(ic.TOPDOWNLEFTRIGHT | ic.COMPLETESCANLINES | ic.SINGLEFRAME
		| ic.SINGLEPASS);
    ic.setDimensions(width, height);
    ic.setColorModel(colorModel);
    ic.setPixels(0, 0, width, height, colorModel, bitmap, 0, width);
    ic.imageComplete(ic.STATICIMAGEDONE);
  }

  public void startProduction(ImageConsumer ic) {
    ic.setHints(ic.TOPDOWNLEFTRIGHT | ic.COMPLETESCANLINES | ic.SINGLEFRAME
		| ic.SINGLEPASS);
    ic.setDimensions(width, height);
    ic.setColorModel(colorModel);
    ic.setPixels(0, 0, width, height, colorModel, bitmap, 0, width);
    ic.imageComplete(ic.STATICIMAGEDONE);
  }
}

