import java.io.*;

public class JISInputStream extends PushbackInputStream implements
KanjiInputStream {
  public static final int ONEBYTE = 0;
  public static final int TWOBYTE = 1;
  public static final int JISC62261978 = 0;
  public static final int JISX02081983 = 1;
  // public static final int JISX02081990 = 2;
  public static final int JISX02121990 = 3;
  public static final int ASCII = 4;
  static final boolean debug = false;
  static final byte CR = (byte) 13;
  static final byte EOL = (byte) 10;
  int mode;
  int type;

  public JISInputStream(InputStream in) {
    super(in);
    mode = ONEBYTE;
    type = JISX02081983;
  }

  public int getMode() { return mode; }

  public int getType() { return type; }

  public byte[] readBytes() throws IOException {
    byte bytes[] = new byte[3];
    while (true) {
      if (mode == ONEBYTE) {
	if (read(bytes, 0, 1) != 1) return null;
	if (bytes[0] == 27) { // start of escape sequence
	  if (read(bytes, 1, 2) != 2) return null;
	  if (bytes[1] == (byte)36) {
	    if (bytes[2] == (byte)64) {
	      type = JISC62261978;
	      mode = TWOBYTE;
	      if (debug)
		System.out.println("JISC-6226-1978\nTWOBYTE mode");
	      continue;
	    }
	    else if (bytes[2] == (byte)66) {
	      type = JISX02081983;
	      mode = TWOBYTE;
	      if (debug)
		System.out.println("JISX-0208-1983\nTWOBYTE mode");
	      continue;
	    }
	    else if (bytes[2] == (byte)40) {
	      if (read(bytes, 0, 1) != 1) return null;
	      if (bytes[0] == (byte) 68) {
		type = JISX02121990;
		mode = TWOBYTE;
		if (debug)
		  System.out.println("JISX-0212-1990\nTWOBYTE mode");
		continue;
	      }
	      else { // unknown escape sequence so return last char read
		System.err.println("unknown escape sequence 1");
		break;
	      }
	    }
	    // else don't know how to process so ignore and read again
	    System.err.println("unknown escape sequence 2");
	    continue;
	  }
	  else if (bytes[1] == (byte) 40) {
	    // in one byte mode
	    if (bytes[2] == (byte) 66) {
	      // one byte ascii mode
	      type = ASCII;
	      mode = ONEBYTE;
	      if (debug)
		System.out.println("ASCII\nONEBYTE mode");
	      continue;
	    }
	  }
	  // else don't know how to process so ignore and read again
	  System.err.println("unknown escape sequence 3 -- value = " + bytes[2]);
	  continue;
	}
	else if (bytes[0] == CR) {
	  // ignore carriage return -- only process end-of-line (EOL)
	  continue;
	}
	else { // read valid one byte char
	  // pad with one byte 0 at the begining
	  bytes[1] = bytes[0];
	  bytes[0] = 0;
	  break;
	}
      }
      else { // two byte mode
	if (read(bytes, 0, 2) != 2) return null;
	if (bytes[0] == (byte) 27) { // start of escape sequence 
	  if (read(bytes, 2, 1) != 1) return null;
	  if (bytes[1] == (byte)40) {
	    if (bytes[2] == (byte)74 || bytes[2] == (byte)72
		|| bytes[2] == (byte)66) {
	      mode = ONEBYTE;
	      if (debug)
		System.out.println("ONEBYTE mode");
	      continue;
	    }
	    else if (bytes[2] == (byte)73) {
	      System.err.println("half-width found");
	      continue;
	    }
	    else {
	      // else don't know how to process so ignore and read again
	      System.err.println("unknown escape sequence 4");
	      continue;
	    }
	  }
	  // else don't know how to process so ignore and read again
	  System.err.println("unknown escape sequence 5");
	  continue;
	}
	else { // valid two-byte character so break
	  break;
	}
      }
    }

    byte returnBytes[] = null;
    /*
    if (mode == ONEBYTE) {
      returnBytes = new byte[1];
      System.arraycopy(bytes, 0, returnBytes, 0, 1);
    }
    else { // two byte mode
      returnBytes = new byte[2];
      System.arraycopy(bytes, 0, returnBytes, 0, 2);
    }
    */
    returnBytes = new byte[2];
    System.arraycopy(bytes, 0, returnBytes, 0, 2);
    return returnBytes;
  }

  public void unread(byte bytes[]) {
    try {
      for (int i=0; i<bytes.length; i++)
	unread((int)bytes[i]);
    }
    catch(IOException ioe) {
      System.err.println("unable to pushback bytes to JISInputStream");
    }
  }

}
