import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

class RomajiTextField extends TextField {
  String oldText;
  int pos;

  public RomajiTextField() { super(); initialize(); }
  public RomajiTextField(String string) { super(string); initialize(); }
  public RomajiTextField(int cols) { super(cols); initialize(); }
  public RomajiTextField(String string, int cols) { super(string, cols); initialize(); }

  public void initialize() {
    //addTextListener(this);
    //addKeyListener(this);
    oldText = null;
    this.enableEvents(AWTEvent.KEY_EVENT_MASK | AWTEvent.TEXT_EVENT_MASK);
  }

  public void processTextEvent(TextEvent e) {
    if (oldText != null) {
      String tempText = oldText;
      oldText = null;
      setText(tempText);
      setCaretPosition(pos);
    }
    else
      super.processTextEvent(e);
  }

  public void processKeyEvent(KeyEvent e) {
    int id = e.getID();
    if (id == KeyEvent.KEY_PRESSED) {
      pos = getCaretPosition();
    }
    else if (id == KeyEvent.KEY_TYPED) {
      switch (e.getKeyChar()) {
      case 'l': oldText = getText(); break;
      case 'L': oldText = getText(); break;
      case 'q': oldText = getText(); break;
      case 'Q': oldText = getText(); break;
      case 'x': oldText = getText(); break;
      case 'X': oldText = getText(); break;
      default: break;
      }
    }
    super.processKeyEvent(e);
  }
}

public class KanaInput extends Panel implements TextListener, KeyListener {
  TextField romaji;
  TextCanvas kana;
  Label rLabel;
  Label kLabel;

  public KanaInput() {
    romaji = new RomajiTextField();
    kana = new FontCanvas(FontCanvas.LEFT, 18);
    kana.setFont(new Font("serif", Font.PLAIN, 18));
    kana.setBackground(Color.lightGray);
    kana.setForeground(Color.black);
    kLabel = new Label("kana:", Label.RIGHT);
    rLabel = new Label("romaji:", Label.RIGHT);
    setLayout(new GridBagLayout());
    Tools.constrain(this, kLabel, 0, 0, 1, 1,
		    GridBagConstraints.NONE, GridBagConstraints.SOUTHEAST,
		    0.0, 0.5, 2, 2, 2, 2);
    Tools.constrain(this, kana, 1, 0, 4, 1,
		    GridBagConstraints.HORIZONTAL, GridBagConstraints.SOUTHWEST,
		    1.0, 0.5, 2, 2, 2, 2);
    Tools.constrain(this, rLabel, 0, 1, 1, 1,
		    GridBagConstraints.NONE, GridBagConstraints.NORTHEAST,
		    0.0, 0.5, 2, 2, 2, 2);
    Tools.constrain(this, romaji, 1, 1, 4, 1,
		    GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
		    1.0, 0.5, 2, 2, 2, 2);
    kana.setFont(new Font("serif", Font.PLAIN, 18));
    romaji.addTextListener(this);
    romaji.addKeyListener(this);
  }

  public KanaInput(TextCanvas textCanvas) {
    romaji = new RomajiTextField();
    kana = textCanvas;
    kana.setBackground(Color.white);  // char background is white so make whole component white
    kana.setForeground(Color.black);
    kLabel = new Label("kana:", Label.RIGHT);
    rLabel = new Label("romaji:", Label.RIGHT);
    setLayout(new GridBagLayout());
    Tools.constrain(this, kLabel, 0, 0, 1, 1,
		    GridBagConstraints.NONE, GridBagConstraints.SOUTHEAST,
		    0.0, 0.5, 2, 2, 2, 2);
    Tools.constrain(this, kana, 1, 0, 4, 1,
		    GridBagConstraints.HORIZONTAL, GridBagConstraints.SOUTHWEST,
		    1.0, 0.5, 2, 2, 2, 2);
    Tools.constrain(this, rLabel, 0, 1, 1, 1,
		    GridBagConstraints.NONE, GridBagConstraints.NORTHEAST,
		    0.0, 0.5, 2, 2, 2, 2);
    Tools.constrain(this, romaji, 1, 1, 4, 1,
		    GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHWEST,
		    1.0, 0.5, 2, 2, 2, 2);
    kana.setFont(new Font("serif", Font.PLAIN, 18));
    romaji.addTextListener(this);
    romaji.addKeyListener(this);
  }

  public String getKana() { return kana.getText(); }

  public String getRomaji() { return romaji.getText(); }

  public void setKana(String text) { kana.setText(text); }

  public void setRomaji(String text) { romaji.setText(text); }

  public void setFieldsForeground(Color color) {
    kana.setForeground(color);
    romaji.setForeground(color);
  }

  public void clear() { romaji.setText(""); kana.setText(""); }

  //handle the text value changes of the romaji text field
  public void textValueChanged(TextEvent e) {
    String curRoma = romaji.getText();
    String rkana = Romaji2Kana.getKana(curRoma);
    if (rkana != null) {
      if (!(curRoma.startsWith("n") || curRoma.startsWith("N")) || curRoma.length()>1) {
	kana.setText(kana.getText() + rkana);
	romaji.setText("");
      }
    }
    else if (curRoma.length()>1) {
      if (curRoma.startsWith("n") || curRoma.startsWith("N")) {
	kana.setText(kana.getText() + Romaji2Kana.getKana(curRoma.substring(0,1)));
	int pos = romaji.getCaretPosition();
	romaji.setText(curRoma.substring(1,curRoma.length()-0));
	romaji.setCaretPosition(pos-1);
      }
      else if (curRoma.charAt(0) == curRoma.charAt(1)) {
	if (Character.isLowerCase(curRoma.charAt(0)))
	  kana.setText(kana.getText() + Romaji2Kana.getKana("ll"));
	else
	  kana.setText(kana.getText() + Romaji2Kana.getKana("LL"));
	int pos = romaji.getCaretPosition();
	romaji.setText(curRoma.substring(1,curRoma.length()-0));
	romaji.setCaretPosition(pos-1);
      }
    }
  }

  //handle the key presses of the romaji text field
  public void keyTyped(KeyEvent e) {}
  public void keyPressed(KeyEvent e) {
    if (romaji.getText().equals("") && e.getKeyChar() == '\b') {
      String ktext = kana.getText();
      kana.setText(ktext.substring(0, ktext.length()-1));
    }
  }
  public void keyReleased(KeyEvent e) {}

  public static void main(String[] argv) {
    try {
      Romaji2Kana.read(new FileInputStream("roma2kana.obj"));
      Frame f = new Frame();
      f.setLayout(new GridBagLayout());
      KanaInput kInput = new KanaInput();
      Tools.constrain(f, kInput, 0, 0, 1, 1,
		      GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		      1.0, 1.0, 2, 2, 2, 2);
      f.setSize(400,100);
      f.setVisible(true);
    }
    catch(FileNotFoundException fnfe) {
      Tools.perr(fnfe);
    }
  }
}
