/* This code is copyrighted (c) 1997 by Marshall Ramsey */
import java.util.*;
import java.io.*;

public class Lesson {
  static final boolean debug = false;
  Vector origList;
  Vector items;
  Vector wrongList;
  int numRight;
  int numWrong;
  int curSize;

  public Lesson(InputStreamReader in) {
    origList = new Vector(50);
    items = new Vector(50);
    wrongList = new Vector(50);
    numRight = 0;
    numWrong = 0;
    parseLesson(read(in));
    curSize = origList.size();
  }

  public Lesson(String string) {
    origList = new Vector(50);
    items = new Vector(50);
    wrongList = new Vector(50);
    numRight = 0;
    numWrong = 0;
    parseLesson(string);
    curSize = origList.size();
  }

  public void appendLesson(InputStreamReader in) {
    parseLesson(read(in));
    curSize = origList.size();
  }

  public void appendLesson(String string) {
    parseLesson(string);
    curSize = origList.size();
  }

  private String read(InputStreamReader in) {
    char[] buf = new char[5120];
    String string = new String();
    int count;
    try {
      while((count = in.read(buf, 0, buf.length)) != -1) {
	string += new String(buf, 0, count);
      }
    }
    catch(IOException ioe) { // ignore
    }
    return string;
  }

  private void parseLesson(String string) {
    StringTokenizer strTok = new StringTokenizer(string, "\n");
    while (strTok.hasMoreTokens()) {
      LessonItem item = new LessonItem(strTok.nextToken());
      if (item.isValid()) {
	origList.addElement(item);
	items.addElement(item);
      }
      else
	break;
    }
  }

  public void randomSubset(int count) {
    items.removeAllElements();
    wrongList.removeAllElements();
    Vector temp = (Vector) origList.clone();
    int tempSize = temp.size();
    for (int i=0; i<tempSize && i<count; i++) {
      int index = (int) (temp.size() * Math.random());
      items.addElement(temp.elementAt(index));
      temp.removeElementAt(index);
    }
    numRight = 0;
    numWrong = 0;
    curSize = items.size();
  }

  public LessonItem nextItem() {
    if (items.size() > 0) {
      int index = (int) (items.size() * Math.random());
      LessonItem returnItem = (LessonItem) items.elementAt(index);
      items.removeElementAt(index);
      return returnItem;
    }
    else if (wrongList.size() > 0) {
      items.removeAllElements();
      Vector temp = items;
      items = wrongList;
      wrongList = temp;
      return nextItem();
    }
    else
      return null;
  }

  public void reset() {
    randomSubset(curSize);
  }

  public void hardReset() {
    items.removeAllElements();
    wrongList.removeAllElements();
    for (int i=0; i<origList.size(); i++)
      items.addElement(origList.elementAt(i));
    numRight = 0;
    numWrong = 0;
    curSize = items.size();
  }

  public void markRight(LessonItem rightItem) {
    numRight++;
  }

  public void markWrong(LessonItem wrongItem) {
    wrongList.addElement(wrongItem);
    numWrong++;
  }

  public int getNumRight() { return numRight; }

  public int getNumWrong() { return numWrong; }

  public int getNumTotal() { return (numRight + numWrong); }

  public double getPercent() {
    if (numRight + numWrong == 0)
      return 1.0;
    else
      return (double)numRight / (double)(numRight + numWrong);
  }

  public int getSize() { return origList.size(); }

  public int getCurSize() { return curSize; }

  // These methods were added for compatibility with LessonWithMemory class
  public void appendLesson(InputStreamReader in, String wheelFileName) { appendLesson(in); }
  public void appendLesson(String input, String wheelFileName) { appendLesson(input); }
  public void setReviewMode(boolean mode) { } // ignore
  public void clearHistory() { } // ignore
  public void save() { } // ignore
}
