import java.awt.*;
import java.util.*;
import java.io.*;

class Romaji2KanaElement implements Serializable {
  public String romaji;
  public String kana;
}

public class Romaji2Kana extends TextField {

  static Hashtable lookupTable = null;

  public static boolean isLoaded() { return (lookupTable != null); }

  public static void read(InputStream in) {
    try {
      ObjectInputStream objIn = new ObjectInputStream(in);
      lookupTable = (Hashtable) objIn.readObject();
    }
    catch(IOException ioe) {
      lookupTable = null;
      Tools.perr(ioe);
    }
    catch(ClassNotFoundException cnfe) {
      lookupTable = null;
      Tools.perr(cnfe);
    }
  }

  public static String getKana(String romaji) {
    if (lookupTable != null)
      return (String) lookupTable.get(romaji);
    else
      return null;
  }

  public static void create(InputStream in) {
    try {
      InputStreamReader reader = new InputStreamReader(in, "EUCJIS");
      parse(read(reader));
    }
    catch(IOException ioe) {
      lookupTable = null;
      Tools.perr(ioe);
    }
  }

  public static void write(OutputStream out) {
    try {
      ObjectOutputStream objOut = new ObjectOutputStream(out);
      objOut.writeObject(lookupTable);
    }
    catch(IOException ioe) {
      Tools.perr(ioe);
    }
  }

  private static String read(InputStreamReader in) {
    char[] buf = new char[5120];
    String string = new String();
    int count;
    try {
      while((count = in.read(buf, 0, buf.length)) != -1) {
	string += new String(buf, 0, count);
      }
    }
    catch(IOException ioe) { // ignore
    }
    return string;
  }

  public static void parse(String convData) {
    try {
      StringTokenizer strTok = new StringTokenizer(convData, " \n\r");
      lookupTable = new Hashtable();
      while (strTok.hasMoreTokens()) {
	String romaji = strTok.nextToken();
	String kana = strTok.nextToken();
	if (lookupTable.containsKey(romaji))
	  System.out.println(romaji + " is duplicate");
	else
	  lookupTable.put(romaji, kana);
      }
    }
    catch(Exception e) { }
  }

  public static void main(String[] argv) {
    try {
      FileInputStream in = new FileInputStream("roma2kana.euc");
      FileOutputStream out = new FileOutputStream("roma2kana.obj");
      Romaji2Kana.create(in);
      Romaji2Kana.write(out);
      in.close();
      out.close();
      System.out.println("wrote " + Romaji2Kana.lookupTable.size() + " elements");
      /*
      FileInputStream in = new FileInputStream("roma2kana.obj");
      Romaji2Kana.read(in);
      in.close();
      System.out.println("read " + Romaji2Kana.lookupTable.size() + " elements");
      /*
      FileOutputStream eucout = new FileOutputStream("checkfile.euc");
      OutputStreamWriter outwriter = new OutputStreamWriter(eucout, "EUCJIS");
      Enumeration elements = Romaji2Kana.lookupTable.elements();
      Enumeration keys = Romaji2Kana.lookupTable.keys();
      while (elements.hasMoreElements() && keys.hasMoreElements()) {
	//String key = (String)keys.nextElement() + " ";
	String el  = (String)elements.nextElement() + "\n";
	//outwriter.write(key, 0, key.length());
	outwriter.write(el, 0, el.length());
	//System.out.println(key);
      }
      outwriter.close();
      eucout.close();
      */
      if (Romaji2Kana.getKana("ka") != null)
	System.out.println("found ka!");
      else
	System.out.println("did not find ka!");
    }
    catch(IOException ioe) {
      Tools.perr(ioe);
    }
  }

}
