/* This code is copyrighted (c) 1997 by Marshall Ramsey */
import java.util.*;
import java.io.*;

public class RouletteWheel implements Serializable {

  static final double DEFAULTODDS = 10.0;
  double defaultOdds;
  double totalOdds;
  Vector vector;
  double minOdds;

  public RouletteWheel() {
    vector = new Vector();
    totalOdds = 0.0;
    minOdds = 0.0;
  }

  public RouletteWheel(double minOdds, double defaultOdds) {
    vector = new Vector();
    totalOdds = 0.0;
    this.minOdds = minOdds;
    this.defaultOdds = defaultOdds;
  }

  public RouletteWheel(int capacity) {
    vector = new Vector(capacity);
    totalOdds = 0.0;
    minOdds = 0.0;
    defaultOdds = DEFAULTODDS;
  }

  public synchronized void addElement() {
    vector.addElement(new Double(defaultOdds));
    totalOdds += defaultOdds;
  }

  public synchronized void addElement(double odds) {
    if (odds < minOdds)
      odds = minOdds;
    vector.addElement(new Double(odds));
    totalOdds += odds;
  }

  public synchronized double getOddsForElementAt(int i) {
    return ((Double)vector.elementAt(i)).doubleValue();
  }

  public synchronized int getRandomElement() {
    double spin = Math.random() * totalOdds;
    double cumOdds = 0.0;
    for (int i=0; i<vector.size(); i++) {
      double curOdds = ((Double)vector.elementAt(i)).doubleValue();
      cumOdds += curOdds;
      if (cumOdds > spin)
	return i;
    }
    return -1; // signifies error
  }

  public synchronized void removeElementAt(int i) throws ArrayIndexOutOfBoundsException {
    double odds = ((Double)vector.elementAt(i)).doubleValue();
    vector.removeElementAt(i);
    totalOdds -= odds;
  }

  public synchronized void modifyOdds(int i, double chgAmt)
       throws ArrayIndexOutOfBoundsException {
    double odds = ((Double)vector.elementAt(i)).doubleValue();
    if (odds + chgAmt > minOdds) {
      odds += chgAmt;
      totalOdds += chgAmt;
    }
    else {
      totalOdds -= odds - minOdds;
      odds = minOdds;
    }
    vector.setElementAt(new Double(odds), i);
  }

  public int size() { return vector.size(); }

  public double getTotalOdds() { return totalOdds; }

  public void addWheel(RouletteWheel rw) {
    for (int i=0; i<rw.size(); i++) {
      addElement(((Double)rw.vector.elementAt(i)).doubleValue());
    }
  }

  /*
  static public void main(String[] args) {
    RouletteWheel rw1 = null;
    RouletteWheel rw2 = null;

    rw1 = new RouletteWheel();
    rw2 = new RouletteWheel();
    rw1.addElement();
    rw1.addElement();
    rw1.addElement();
    rw1.removeElementAt(1);
    rw1.modifyOdds(1, 100);
    rw2.addElement();
    rw2.addElement();
    rw2.addElement();
    rw2.addWheel(rw1);

    try {
      String fname = "test.obj";

      FileOutputStream fout = new FileOutputStream(fname);
      ObjectOutputStream out = new ObjectOutputStream(fout);
      out.writeObject(rw2);
      fout.close();

      FileInputStream fin = new FileInputStream(fname);
      ObjectInputStream in = new ObjectInputStream(fin);
      rw2 = (RouletteWheel) in.readObject();
      fin.close();
    }
    catch(IOException ioe) {
      Tools.perr(ioe);
    }
    catch(ClassNotFoundException cnfe) {
      Tools.perr(cnfe);
    }

    for (int i=0; i<20; i++)
      System.out.println(rw2.getRandomElement());
  }
  */
}
